#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#ifdef __FreeBSD__
#  include <machine/soundcard.h>
#  include <awe_voice.h>
#elif defined(linux)
#  include <linux/soundcard.h>
#  include <linux/awe_voice.h>
#endif

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "data.h"
#include "awebank.h"
#include "seq.h"
#include "util.h"
#include "sfopts.h"
#include "fx.h" 		// Chorus and reverb fx data

void LoadSoundFont(char *s, int bank, int chorus, int reverb)
{
    GtkWidget *progressbar;
    LoadList *part_list = NULL;
    int rc;

    progressbar = lookup_widget(window1,"progressbar1");
    /*
    gtk_progress_set_format_string(GTK_PROGRESS(progressbar), "Loading ...");
    while (gtk_events_pending())
        gtk_main_iteration();
    */

    awe_option.default_bank = bank;
    awe_option.default_chorus = chorus;
    awe_option.default_reverb = reverb;

    seq_remove_samples();
    while (gtk_events_pending())
        gtk_main_iteration();
    if (bank==0)
    {
        seq_reset_samples();
        rc = awe_load_bank(s,part_list,TRUE);
    }
    else
        rc = awe_load_bank(s,part_list,FALSE);

    switch (rc) {
    case AWE_RET_OK:
        break;
    case AWE_RET_NOMEM:
        msgbox1 = create_messagebox1("You do not have enough memory left on your AWE32.");
        gtk_widget_show(msgbox1);
        gtk_grab_add(msgbox1);
        break;
    case AWE_RET_NOT_FOUND:
        msgbox1 = create_messagebox1("The file you specified does not exists.");
        gtk_widget_show(msgbox1);
        gtk_grab_add(msgbox1);
        break;
    };

//    gtk_progress_set_format_string(GTK_PROGRESS(progressbar), _("%VKb of %UKb free"));
    gtk_progress_set_value(GTK_PROGRESS(progressbar), seq_mem_avail()/1024.0);
}


/* Main Window */
void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


/* Load SoundFont */
void
on_button1_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    window3 = create_window3();
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(window3, "spinbutton4")),Preferences.chorus);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(window3, "spinbutton5")),Preferences.reverb);
    gtk_widget_show(window3);
    gtk_grab_add(window3);
}


/* Load Default */
void
on_button2_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    LoadSoundFont(Preferences.defaultsf, 0,
                  Preferences.chorus, Preferences.reverb);
}


/* Preferences button */
void
on_button3_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    window2 = create_window2();
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(window2, "combo_entry3")),
		       Preferences.defaultsf);
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(window2, "entry1")),
		       Preferences.sfpath);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
				 (lookup_widget(window2, "togglebutton1")),
                                 Preferences.values);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(window2, "spinbutton1")),Preferences.chorus);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(window2, "spinbutton2")),Preferences.reverb);

    gtk_widget_show(window2);
    gtk_grab_add(window2);
}


/* Quit button */
void
on_button4_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


/* Treble slider */
gboolean
on_hscale1_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    GtkAdjustment *tadj, *badj;

    tadj = gtk_range_get_adjustment(GTK_RANGE(lookup_widget(window1,"hscale1")));
    badj = gtk_range_get_adjustment(GTK_RANGE(lookup_widget(window1,"hscale2")));
    seq_set_equalizer(tadj->value,badj->value);

    return FALSE;
}


/* Bass slider */
gboolean
on_hscale2_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    GtkAdjustment *tadj, *badj;

    tadj = gtk_range_get_adjustment(GTK_RANGE(lookup_widget(window1,"hscale1")));
    badj = gtk_range_get_adjustment(GTK_RANGE(lookup_widget(window1,"hscale2")));
    seq_set_equalizer(tadj->value,badj->value);

    return FALSE;
}


/* Chorus setting */
void
on_combo_entry1_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
    char *tmp = gtk_editable_get_chars(editable,0,-1);
    int i;

    i = 0;
    while (i<AWE_CHORUS_NUMBERS)
    {
        if (strcmp(tmp,CLT[i])==0)
            break;
        i++;
    }
    if (i<AWE_CHORUS_NUMBERS)
        seq_set_chorus(i);
}


/* Reverb setting */
void
on_combo_entry2_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
    char *tmp = gtk_editable_get_chars(editable,0,-1);
    int i;

    i = 0;
    while (i<AWE_REVERB_NUMBERS)
    {
        if (strcmp(tmp,RLT[i])==0)
            break;
        i++;
    }
    if (i<AWE_REVERB_NUMBERS)
        seq_set_reverb(i);
}


/* OK button in window2 (=Preferences) */
void
on_button6_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    FILE *f;
    char tmp[256], *homeenv;
    
    strcpy(Preferences.defaultsf,
	   gtk_entry_get_text(GTK_ENTRY
			      (lookup_widget(window2, "combo_entry3"))));
    strcpy(Preferences.sfpath,
	   gtk_entry_get_text(GTK_ENTRY(lookup_widget(window2, "entry1"))));
    Preferences.values =
	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
				     (lookup_widget
                                      (window2, "togglebutton1")));

    Preferences.chorus = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(window2, "spinbutton1")));
    Preferences.reverb = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(window2, "spinbutton2")));

    homeenv = getenv("HOME");
    strcpy(tmp, homeenv);
    strcat(tmp, "/.awecp");
    f = fopen(tmp, "wt");
    if (!f)
        printf("Error creating \"~/.awecp\"\n");
    else
    {
        fprintf(f,"defaultsf=%s\n",Preferences.defaultsf);
        fprintf(f,"sfpath=%s\n",Preferences.sfpath);
        fprintf(f,"chorus=%d\n",Preferences.chorus);
        fprintf(f,"reverb=%d\n",Preferences.reverb);
        fprintf(f,"values=%d\n",Preferences.values);
        fclose(f);
    }

    if (Preferences.values)
    {
	gtk_scale_set_draw_value(GTK_SCALE(lookup_widget(window1, "hscale1")),
				 TRUE);
	gtk_scale_set_draw_value(GTK_SCALE(lookup_widget(window1, "hscale2")),
				 TRUE);
    }
    else
    {
	gtk_scale_set_draw_value(GTK_SCALE(lookup_widget(window1, "hscale1")),
				 FALSE);
	gtk_scale_set_draw_value(GTK_SCALE(lookup_widget(window1, "hscale2")),
				 FALSE);
    }
    chdir(Preferences.sfpath);
    if (errno!=0)
        printf("Error on chdir(%s)\n",Preferences.sfpath);

    gtk_grab_remove(window2);
    gtk_widget_hide(window2);
    gtk_widget_destroy(window2);
}


/* Cancel button in window2 (=Preferences) */
void
on_button7_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_grab_remove(window2);
    gtk_widget_hide(window2);
    gtk_widget_destroy(window2);
}


/* OK in window3 (=load SoundFont) */
void
on_button8_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    char tmp[256];
    int b, c, r;
    
    strcpy(tmp,gtk_entry_get_text(GTK_ENTRY(
                                  lookup_widget(window3, "combo_entry4"))));

    b = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(window3, "spinbutton3")));
    c = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(window3, "spinbutton4")));
    r = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(window3, "spinbutton5")));

    gtk_grab_remove(window3);
    gtk_widget_hide(window3);
    gtk_widget_destroy(window3);

    if (strcmp(tmp,""))
        LoadSoundFont(tmp,b,c,r);
}


/* Cancel in window3 (=load SoundFont) */
void
on_button9_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_grab_remove(window3);
    gtk_widget_hide(window3);
    gtk_widget_destroy(window3);
}


/* OK in messagebox1 */
void
on_button10_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_grab_remove(msgbox1);
    gtk_widget_hide(msgbox1);
    gtk_widget_destroy(msgbox1);
}
